from __future__ import annotations

import logging
import os
from pathlib import Path

import natsort
import unidecode


def create_thumb(file_path):
    """Create thumbnail from video file path using ffmpeg CLI

    Args:
        file_path (str): video file path

    Returns:
        str: created thumbnail file path
    """

    path_dir = os.path.split(file_path)[0]
    file_name = os.path.basename(file_path)
    file_name_without_extension = os.path.splitext(file_name)[0]
    file_name_output = file_name_without_extension + ".jpg"
    file_path_output = os.path.join(path_dir, file_name_output)
    if os.path.exists(file_path_output):
        os.remove(file_path_output)
    # create thumb
    os.system(
        f'ffmpeg -v quiet -stats -y -i "{file_path}" -vframes 1 "{file_path_output}"'
    )
    return file_path_output


def create_txt(file_path, stringa):

    file = open(file_path, "w", encoding="utf8")
    file.write(stringa)
    file.close()


def get_txt_content(file_path):

    list_encode = ["utf-8", "ISO-8859-1"]  # utf8, ANSI
    for encode in list_encode:
        try:
            file = open(file_path, "r", encoding=encode)
            file_content = file.readlines()
            file_content = "".join(file_content)
            file.close()
            return file_content
        except:
            continue

    raise Exception("encode", f"Cannot open file: {file_path}")


def get_all_file_path(folder_path: Path, sort=True) -> dict[str, list[Path]]:
    """Returns List of all file paths inside a folder, recursively.
    Option to Sort naturally.

    Args:
    -----
        folder_path (Path): folder path
        sort (bool, optional): Return classified. Defaults to True.

    Returns:
    --------
        dict[str, list[Path]]: keys: ['content', 'errors']. values: list[Path]
    """

    def iter_folder(
        sub_folder,
        list_file_path: list[Path] = [],
        list_error: list[Path] = [],
    ) -> tuple[list[Path], list[Path]]:

        for x in sub_folder.iterdir():
            if not x.exists():
                logging.error("path_too_long: %s", x)
                list_error.append(x)
                continue
            if x.is_dir():
                iter_folder(x, list_file_path, list_error)
            else:
                list_file_path.append(x)
        return list_file_path, list_error

    if not folder_path.exists():
        logging.error("Folder not exists: %s", folder_path)
        raise FileNotFoundError(f"Folder not exists: {folder_path}")

    list_file_path, list_error = iter_folder(folder_path)

    if sort:
        list_file_path = natsort.natsorted(
            list_file_path, lambda x: unidecode.unidecode(str(x).lower())
        )
        list_error = natsort.natsorted(
            list_error, lambda x: unidecode.unidecode(str(x).lower())
        )

    return {"content": list_file_path, "errors": list_error}



"""Console script for tgsender."""
import sys

import click

from . import tgsender


@click.command()
def main(args=None):
    """Console script for tgsender."""
    click.echo(
        "Replace this message by putting your code into " "tgsender.cli.main"
    )
    click.echo("See click documentation at https://click.palletsprojects.com/")

    tgsender.main()
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import logging
import os
import sys
import time
from datetime import datetime
from pathlib import Path

from pyrogram import Client, types

from .. import utils
from ..mediainfo import ffprobe


def logging_config():
    log_file_name = "api_telegram"
    logfilename = "log-" + log_file_name + ".txt"
    logging.basicConfig(
        level=logging.INFO,
        format=" %(asctime)s-%(levelname)s-%(message)s",
        handlers=[logging.FileHandler(logfilename, "w", "utf-8")],
    )
    # set up logging to console
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    # set a format which is simpler for console use
    formatter = logging.Formatter(" %(asctime)s-%(levelname)s-%(message)s")
    console.setFormatter(formatter)
    # add the handler to the root logger
    logging.getLogger("").addHandler(console)


def ensure_connection():

    if Path("user.session").exists():
        try:
            with Client(
                "user", workdir=Path("user.session").absolute().parent
            ) as app:
                message = app.send_message(
                    "me", text="telegram filesender-Connected..."
                )
                message.delete()
            return
        except:
            print("Delete Session file and try again.")

    while True:
        try:
            api_id = int(input("Enter your api_id: "))
            api_hash = input("Enter your api_hash: ")

            with Client(
                "user",
                api_id,
                api_hash,
                workdir=Path("user.session").absolute().parent,
            ) as app:
                message = app.send_message(
                    "me", text="telegram filesender-Connected..."
                )
                message.delete()
            return
        except:
            print("\nError. Try again.\n")
            pass


# Keep track of the progress while uploading
def progress(current, total):

    stringa = f"{current * 100 / total:.1f}%"
    sys.stdout.write(stringa)
    sys.stdout.flush()
    sys.stdout.write("\b" * len(stringa))
    sys.stdout.flush()


def send_video(chat_id, file_path, caption):

    logging.info("Sending video...")
    try:
        metadata = ffprobe(file_path).get_output_as_dict()
        metadata_streams = metadata["streams"]
        video_metadata = metadata_streams[0]
    except:
        # case of error, show all file metadata
        print(file_path)
        print(ffprobe(file_path).get_output_as_dict())
        metadata = ffprobe(file_path).get_output_as_dict()
        metadata_streams = metadata["streams"]
        video_metadata = metadata_streams[0]
    try:
        width = video_metadata["width"]
    except:
        video_metadata = metadata_streams[1]
    try:
        width = video_metadata["width"]
        height = video_metadata["height"]
        duration = int(float(metadata["format"]["duration"]))
    except Exception as e:
        logging.error(f"File Error: {file_path}.\napi_telegram.py line 63")
        raise ValueError(e)
    thumb = utils.create_thumb(file_path)

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_video(
            chat_id,
            file_path,
            caption=caption,
            progress=progress,
            supports_streaming=True,
            width=width,
            height=height,
            duration=duration,
            thumb=thumb,
        )
    os.remove(thumb)
    return return_


def send_sticker(chat_id, sticker):

    logging.info("Sending sticker...")
    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_sticker(chat_id, sticker)
    return return_


def send_photo(chat_id, file_path, caption):

    logging.info("Sending photo...")
    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_photo(
            chat_id, file_path, caption=caption, progress=progress
        )
    return return_


def send_audio(chat_id, file_path, caption):
    logging.info("Sending audio...")
    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_audio(
            chat_id, file_path, caption=caption, progress=progress
        )
    return return_


def send_document(chat_id, file_path, caption):
    logging.info("Sending document...")
    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_document(
            chat_id, file_path, caption=caption, progress=progress
        )
    return return_


def send_message(chat_id, text):

    logging.info("Sending message...")
    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_message(
            chat_id, text=text, disable_web_page_preview=True
        )
    return return_


def pin_chat_message(chat_id, message_id):

    logging.info("Pinning message...")
    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.pin_chat_message(
            chat_id, message_id=message_id, both_sides=True
        )
    return return_


def get_messages(chat_id, message_ids):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.get_messages(chat_id, message_ids)
    return return_


def get_history(chat_id):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.get_history(chat_id)
    return return_


def get_list_media_doc(list_dict_sent_doc):
    """
    Generates a list of "inputmediadocument"
     from a list of 'return of sent files'.
    Used to forward files in album format with 'send_media_group' method

    Args:
        list_dict_sent_doc (list):
            List of returns from many files sent by the send_document method

    Returns:
        list: list of InputMediaDocument pyrogram type,
               necessary to send_media_group method
    """

    list_media_doc = []
    for dict_sent_doc in list_dict_sent_doc:
        file_id = dict_sent_doc["file_id"]
        caption = dict_sent_doc["caption"]
        media = types.InputMediaDocument(media=file_id, caption=caption)

        list_media_doc.append(media)
    return list_media_doc


def send_media_group(chat_id, list_media):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.send_media_group(chat_id, media=list_media)
    return return_


def delete_messages(chat_id, list_message_id):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.delete_messages(
            chat_id=chat_id, message_ids=list_message_id
        )
    return return_


def send_file(dict_file_data, chat_id, time_limit=99):

    file_path = dict_file_data["file_output"]
    description = dict_file_data["description"]
    file_extension = Path(file_path).suffix
    if file_extension == ".mp4":
        type_file = "video"
    elif file_extension == ".mp3":
        type_file = "audio"
    elif file_extension in [".png", ".jpg", ".jpeg", ".gif"]:
        type_file = "photo"
    else:
        type_file = "document"

    if type_file == "video":
        return_ = send_video(
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )
    elif type_file == "audio":
        return_ = send_audio(
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )
    elif type_file == "photo":
        return_ = send_photo(
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )
    elif type_file == "document":
        return_ = send_document(
            chat_id=chat_id,
            file_path=file_path,
            caption=description,
        )

    return return_


def send_files(list_dict, chat_id, time_limit=20):
    """Sends a series of files to the same chat_id

    Args:
        list_dict (list): list of dict. dict with keys:
            file_path=Absolute file_path
            description=file description
            file_output=file name for log
    """

    list_return = []
    len_list_dict = len(list_dict)
    for index, d in enumerate(list_dict):
        order = index + 1
        file_path = d["file_output"]

        if not Path(file_path).exists():
            logging.error(f"file not exist. {file_path}")
            continue

        now = datetime.now()
        dt_string = now.strftime("%Y/%m/%d %H:%M:%S")
        print(f"{dt_string}-{order}/{len_list_dict} Uploading: {file_path}")
        logging.info(f"{order}/{len_list_dict} Uploading: {file_path}")

        file_extension = Path(file_path).suffix
        description = d["description"]

        if file_extension == ".mp4":
            type_file = "video"
        elif file_extension == ".mp3":
            type_file = "audio"
        elif file_extension in [".png", ".jpg", ".jpeg", ".gif"]:
            type_file = "photo"
        else:
            type_file = "document"

        while True:
            try:
                if type_file == "video":
                    return_ = send_video(
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                elif type_file == "audio":
                    return_ = send_audio(
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                elif type_file == "photo":
                    return_ = send_photo(
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                elif type_file == "document":
                    return_ = send_document(
                        chat_id=chat_id,
                        file_path=file_path,
                        caption=description,
                    )
                break
            except Exception as e:
                print(e)
                print("\nError. Trying again...")
                time.sleep(30)
                continue
        list_return.append(return_)
    return list_return


def create_channel(title, description):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_chat = app.create_channel(title=title, description=description)
    chat_id = return_chat.id
    return chat_id


def add_chat_members(chat_id, user_ids):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_chat = app.add_chat_members(chat_id=chat_id, user_ids=user_ids)


def promote_chat_members(chat_id, user_ids):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:

        privileges_config = types.ChatPrivileges(
            can_change_info=True,
            can_post_messages=True,
            can_edit_messages=True,
            can_delete_messages=True,
            can_promote_members=True,
        )

        for user_id in user_ids:
            app.promote_chat_member(
                chat_id=chat_id, user_id=user_id, privileges=privileges_config
            )


def set_chat_description(chat_id, description):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        app.set_chat_description(chat_id=chat_id, description=description)


def export_chat_invite_link(chat_id):

    with Client("user", workdir=Path("user.session").absolute().parent) as app:
        return_ = app.export_chat_invite_link(chat_id=chat_id)

    return return_


def get_template(folder_path_descriptions, file_name):

    template_path = Path(folder_path_descriptions) / file_name
    if not template_path.exists():
        template_path = (
            Path(__file__).absolute().parent.parent / "template" / file_name
        )
    if not template_path.exists():
        raise FileNotFoundError(f"{template_path=}")
    return template_path


def get_channel_title(folder_path_descriptions):

    header_project_name = "header_project.txt"
    header_project_path = get_template(
        folder_path_descriptions, header_project_name
    )
    print(f"\n{header_project_path=}\n")
    channel_info_stringa = utils.get_txt_content(header_project_path)
    list_channel_info = channel_info_stringa.split("\n")

    title = list_channel_info[0]
    return title


def get_channel_description(chat_invite_link, folder_path_descriptions):

    header_template_name = "header_project.txt"
    header_template_path = get_template(
        folder_path_descriptions, header_template_name
    )

    channel_info_stringa = utils.get_txt_content(header_template_path)
    list_channel_info = channel_info_stringa.split("\n")

    description = "\n".join(list_channel_info[1:])
    description = description.replace("{chat_invite_link}", chat_invite_link)
    return description


def get_list_adms(channel_adms: str):

    if not channel_adms:
        return []
    channel_adms_stringa_list = channel_adms.split(",")
    list_adms = []
    for line in channel_adms_stringa_list:
        list_adms.append(line.strip())
    return list_adms



from __future__ import annotations

import logging
import os
import sys
import time
from configparser import ConfigParser
from pathlib import Path

import pandas as pd
import pyautogui as pag
import pyperclip

from . import utils
from .api import (
    add_chat_members,
    create_channel,
    ensure_connection,
    export_chat_invite_link,
    get_channel_description,
    get_channel_title,
    get_list_adms,
    promote_chat_members,
    send_file,
    set_chat_description,
)


def get_config_data(path_file_config):
    """get default configuration data from file config.ini

    Returns:
        dict: config data
    """

    config_file = ConfigParser()
    config_file.read(path_file_config)
    default_config = dict(config_file["default"])
    return default_config


def gen_data_frame(list_file_path: list[Path]):

    list_dict = [
        {"description": file_path.name, "file_output": file_path}
        for file_path in list_file_path
    ]
    df = pd.DataFrame(list_dict)
    list_columns = ["file_output", "description"]
    df = df.reindex(list_columns, axis=1)
    return df


def create_report_descriptions(upload_plan_path):

    str_msg_paste_folder = "Paste the path folder with your files"
    str_msg_paste_here = "Paste here: "
    print(str_msg_paste_folder)
    while True:
        path_folder = input(str_msg_paste_here)
        if Path(path_folder).exists():
            all_file_path = utils.get_all_file_path(Path(path_folder))
            break
        else:
            logging.error("Folder not exist. Try again.")

    list_file_path = all_file_path["content"]
    df = gen_data_frame(list_file_path)
    df.to_csv(upload_plan_path, index=False)


def pag_hotkey(key_1, key_2):

    pag.keyDown(key_1)
    pag.keyDown(key_2)
    pag.keyUp(key_1)
    pag.keyUp(key_2)


def set_win_positions():
    """
    Keep the 'windows explorer' windows in front
    and telegram windows in back
    """

    pag_hotkey("alt", "tab")
    time.sleep(1)
    pag.keyDown("alt")
    pag.press("tab")
    time.sleep(0.5)
    pag.press("tab")
    pag.keyUp("alt")
    time.sleep(0.5)


def change_between_telegram_winexplorer():

    pag_hotkey("alt", "tab")
    time.sleep(2)


def get_next_video_to_send(path_file_description: Path):

    try:
        df = pd.read_csv(path_file_description)
    except Exception as e:
        print(f"Can't open file: {path_file_description}")
        print(e)

    mask_df_to_send = df["sent"].isin([0])
    df_to_send = df.loc[mask_df_to_send, :]
    count_files_to_send = df_to_send.shape[0]
    if count_files_to_send == 0:
        return 0, False
    else:
        file_index = df_to_send.index[0]
        df_to_send = df_to_send.reset_index(drop=True)
        dict_first_line = df_to_send.loc[0, :]
        return file_index, dict_first_line


def update_description_file_sent(path_file_description, dict_file_data):

    try:
        df = pd.read_csv(path_file_description)
    except Exception as e:
        print(f"Can't open file: {path_file_description}")
        print(e)

    file_path = dict_file_data["file_output"]
    mask_file_path = df["file_output"].isin([file_path])
    df_filter = df.loc[mask_file_path, :]
    len_df_filter = len(df_filter)
    if len_df_filter != 1:
        print(f"Find {len_df_filter} line for file: {file_path}")
        sys.exit()
    index_video = df_filter.index
    # update df
    df.loc[index_video, "sent"] = 1
    df.to_csv(path_file_description, index=False)


def ensure_existence_sent_column(df_list, file_path_descriptions):

    description_columns = df_list.columns
    if "sent" not in description_columns:
        df_list["sent"] = 0
        df_list.to_csv(file_path_descriptions, index=False)


def get_data_upload_plan(upload_plan_path_folder: Path) -> list[dict]:

    file_path_upload_plan = upload_plan_path_folder / "upload_plan.csv"
    df_upload_plan = pd.read_csv(file_path_upload_plan)
    data_upload_plan = df_upload_plan.to_dict("records")
    return data_upload_plan


def paste_on_telegram_app(desc):

    time.sleep(2)
    pag_hotkey("ctrl", "v")
    pyperclip.copy(desc)
    time.sleep(3)
    pag_hotkey("ctrl", "v")
    time.sleep(1)
    pag.press("enter")
    time.sleep(1)


def ask_send_app_or_api():

    str_msg_1 = "How do you intend to send the files?"
    str_msg_2 = "1-By telegram desktop app"
    str_msg_3 = "2-By telegram api (default)\n"
    str_msg_answer = "Type the number: "

    print("\n".join([str_msg_1, str_msg_2, str_msg_3]))

    answer = input(str_msg_answer)

    if answer == "":
        return 2

    return int(answer)


def send_via_telegram_app(data_upload_plan):

    set_win_positions()
    qt_files = len(data_upload_plan)
    print(f"Send {qt_files} files")
    time.sleep(0.5)

    for d in data_upload_plan:
        desc = d["description"]
        # copy file
        pag_hotkey("ctrl", "c")

        # change to telegram
        change_between_telegram_winexplorer()

        # paste on telegram app
        paste_on_telegram_app(desc)

        # change back to windows explorer and select next file
        change_between_telegram_winexplorer()
        pag.press("down")
        time.sleep(0.5)


def process_create_channel(folder_path_descriptions):

    # TODO: The channel title must be the name of the original project folder
    title = get_channel_title(folder_path_descriptions)
    description = "channel description"

    chat_id = create_channel(title=title, description=description)
    return chat_id


def config_channel(
    chat_id, chat_invite_link, list_adms, folder_path_descriptions
):

    description = get_channel_description(
        chat_invite_link, folder_path_descriptions
    )
    set_chat_description(chat_id=chat_id, description=description)

    if len(list_adms) != 0:
        add_chat_members(chat_id=chat_id, user_ids=list_adms)
        promote_chat_members(chat_id=chat_id, user_ids=list_adms)


def send_via_telegram_api(folder_path_upload_plan: Path, dict_config: dict):

    file_path_descriptions = folder_path_upload_plan / "upload_plan.csv"
    df_list = pd.read_csv(file_path_descriptions)

    ensure_existence_sent_column(df_list, file_path_descriptions)

    ensure_connection()

    chat_id = process_to_send_telegram(folder_path_upload_plan, dict_config)
    time_limit = int(dict_config["time_limit"])

    # Connection test to avoid infinite loop of asynchronous attempts
    # to send files without pause to fill connection pool

    files_count = df_list.shape[0]
    while True:
        index, dict_file_data = get_next_video_to_send(file_path_descriptions)
        # mark file as sent
        if dict_file_data is False:
            break
        else:
            file_path = dict_file_data["file_output"]
            print(f"{index+1}/{files_count} Uploading: {file_path}")

            send_file(dict_file_data, chat_id, time_limit)

            update_description_file_sent(
                file_path_descriptions, dict_file_data
            )


def test_chat_id(dict_config):

    if "chat_id" in dict_config.keys():
        chat_id = dict_config["chat_id"]
        is_int = isinstance(chat_id, int)
        if is_int:
            if chat_id < 0:
                return True
        print("config['chat_id'] must be negative integer")
        return False
    else:
        print("config['chat_id'] key 'chat_id' not found in config file")
        return False


def save_metadata_file(chat_id, chat_invite_link, file_path_metadata):

    dict_metadata = {"chat_id": chat_id, "chat_invite_link": chat_invite_link}
    utils.create_txt(file_path_metadata, str(dict_metadata))


def process_to_send_telegram(folder_path_descriptions, dict_config):
    """
    Creates a new properly configured channel
    or use the existing chat_id in the configuration file

    Args:
        folder_path_descriptions (str):
            folder path where the upload_plan.csv file is located
        dict_config (dict):
            configuration data.
            template: {chat_id:negative int, create_new_channel:boolean}

    Returns:
        int: chat_id. negative integer
    """

    # check if there is necessary to continue upload
    file_path_metadata = os.path.realpath(
        os.path.join(folder_path_descriptions, "channel_metadata")
    )
    continue_upload = False
    if os.path.exists(file_path_metadata):
        dict_metadata = eval(utils.get_txt_content(file_path_metadata))
        continue_upload = True

    # get chat_id
    # Create new channel-Default True
    if dict_config["create_new_channel"] == 0 or continue_upload:
        channel_new = False
    else:
        channel_new = True

    if channel_new:
        # create new channel
        chat_id = process_create_channel(folder_path_descriptions)
        chat_invite_link = export_chat_invite_link(chat_id=chat_id)
        save_metadata_file(chat_id, chat_invite_link, file_path_metadata)

        # config new channel
        list_adms = get_list_adms(dict_config["channel_adms"])
        config_channel(
            chat_id, chat_invite_link, list_adms, folder_path_descriptions
        )
    else:
        # use existent channel
        if continue_upload:
            dict_data = dict_metadata
        else:
            dict_data = dict_config

        chat_id_is_valid = test_chat_id(dict_data)
        if chat_id_is_valid:
            chat_id = dict_data["chat_id"]

    return chat_id


def ask_create_or_use(upload_plan_path):

    str_msg_create_or_use_1 = "About the upload_plan.csv report"
    str_msg_create_or_use_2 = "1-Use existing (default)"
    str_msg_create_or_use_3 = "2-Create a new one\n"
    str_msg_answer = "Type the number: "

    print(str_msg_create_or_use_1)
    print(str_msg_create_or_use_2)
    print(str_msg_create_or_use_3)

    create_or_use_answer = input(str_msg_answer)
    if create_or_use_answer == "2":
        create_report_descriptions(upload_plan_path)
        input("Report Created. Press a key to continue.")
    else:
        pass


def main():
    """
    An easy and automatic way to post a series of files to the telegram desktop
    app, along with personalized descriptions.

    How to use: https://github.com/apenasrr/Telegram_filesender/blob/master/README.md

    Args:
        upload_plan_path (str):
        dict_config (dict): {create_new_channel_flag: int, chat_id: int}
    """

    path_config_file = Path(__file__).absolute().parent / "config.ini"
    dict_config = get_config_data(path_config_file)

    upload_plan_path = Path("upload_plan.csv").absolute()
    ask_create_or_use(upload_plan_path)

    upload_plan_path_folder = upload_plan_path.parent
    data_upload_plan = get_data_upload_plan(upload_plan_path_folder)
    send_mode = ask_send_app_or_api()

    if send_mode == 1:
        send_via_telegram_app(data_upload_plan)
    elif send_mode == 2:
        send_via_telegram_api(upload_plan_path_folder, dict_config)


if __name__ == "__main__":

    main()



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "Click>=7.0",
    "asyncio",
    "openpyxl",
    "pyperclip",
    "pandas",
    "pyrogram",
    "pyautogui",
    "tgcrypto",
    "natsort",
    "unidecode",
]

test_requirements = ["pytest>=3"]

setup(
    author="apenasrr",
    author_email="apenasrr@gmail.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="An easy and automatic telegram uploader, with personalized descriptions.",
    entry_points={
        "console_scripts": [
            "tgsender=tgsender.cli:main",
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="tgsender",
    name="tgsender",
    packages=find_packages(include=["tgsender", "tgsender.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/apenasrr/tgsender",
    version="0.1.1",
    zip_safe=False,
)



"""
source: https://stackoverflow.com/a/63215561/10681595
"""

import subprocess
import json
from pathlib import Path


class FFProbeResult:

    def __init__(self, return_code: int = None, output: str = '', error: str = '', format=None):
        self.return_code = return_code
        self.output = output
        self.error = error
        self.format = format
        self._output_as_dict = None

    def get_output_as_dict(self):
        if self._output_as_dict is None:
            if self.format == 'json':
                self._output_as_dict = json.loads(self.output)
            elif self.format == 'flat':
                output = [e.split('=') for e in self.output.strip().split('\n')]
                self._output_as_dict = {key_val[0]: key_val[1] for key_val in output}
            else:
                raise ValueError("ffprobe format '%s' not supported to build dict" % self.format)
        return self._output_as_dict

    def to_json_file(self, path: Path, mode='w', **kwargs):
        """
        :param mode: file open mode
        :param kwargs: kwargs for pathlib.Path().open()
        """
        path = path if isinstance(path, Path) else Path(path)
        with path.open(mode, **kwargs) as f:
            json.dump(self.get_output_as_dict(), f, indent=4)
            logger.debug('Dumped ffprobe output into %s', path)


def ffprobe(file_path, ffprobe_format="json", format_optn='', log_level='error') -> FFProbeResult:

    assert ffprobe_format in ['json', 'flat'], "format must be json or flat, not %s" % ffprobe_format
    format_optn = '=' + format_optn if format_optn else format_optn
    command_array = ["ffprobe",
                     "-v", log_level,
                     "-print_format", ffprobe_format + format_optn,
                     '-show_programs',
                     "-show_format",
                     "-show_streams",
                     f"{file_path}"]
    try:
        result = subprocess.run(command_array, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    except Exception as e:
        logger.critical("ffprobe failed to run on %s, with the following error: '%s'\n"
                        " check first that cmd is in your path",
                        file_path, result.stderr, exc_info=True)
        raise e

    return FFProbeResult(return_code=result.returncode,
                         output=result.stdout,
                         error=result.stderr,
                         format=ffprobe_format)